/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.eviction;

import org.jboss.cache.config.Dynamic;

/**
 * Configuration implementation for {@link LFUAlgorithm}.
 *
 * @author Manik Surtani
 * @since 3.0
 */
public class LFUAlgorithmConfig extends EvictionAlgorithmConfigBase
{
   /**
    * The serialVersionUID
    */
   private static final long serialVersionUID = 1865801530398969179L;

   @Dynamic
   private int minNodes;

   public LFUAlgorithmConfig()
   {
      evictionAlgorithmClassName = LFUAlgorithm.class.getName();
   }

   public LFUAlgorithmConfig(int maxNodes, int minNodes)
   {
      this();
      setMaxNodes(maxNodes);
      setMinNodes(minNodes);
   }

   public int getMinNodes()
   {
      return minNodes;
   }

   public void setMinNodes(int minNodes)
   {
      testImmutability("minNodes");
      this.minNodes = minNodes;
   }

   @Override
   public String toString()
   {
      StringBuilder ret = new StringBuilder();
      ret.append("LFUAlgorithmConfig: maxNodes = ").append(getMaxNodes()).append(" minNodes = ").append(getMinNodes());
      return ret.toString();
   }

   @Override
   public boolean equals(Object obj)
   {
      if (obj instanceof LFUAlgorithmConfig && super.equals(obj))
      {
         return (this.minNodes == ((LFUAlgorithmConfig) obj).minNodes);
      }
      return false;
   }

   @Override
   public int hashCode()
   {
      int result = super.hashCode();
      result = 31 * result + minNodes;
      return result;
   }

   @Override
   public LFUAlgorithmConfig clone() throws CloneNotSupportedException
   {
      return (LFUAlgorithmConfig) super.clone();
   }

   @Override
   public void reset()
   {
      super.reset();
      minNodes = -1;
      evictionAlgorithmClassName = LFUAlgorithm.class.getName();
   }
}