/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.eviction;

import org.jboss.cache.config.ConfigurationException;

/**
 * Configuration for {@link MRUAlgorithm}.
 * <p/>
 * Requires a "maxNodes" attribute otherwise a ConfigurationException is thrown.
 *
 * @author Manik Surtani
 * @since 3.0
 */
public class MRUAlgorithmConfig extends EvictionAlgorithmConfigBase
{
   /**
    * The serialVersionUID
    */
   private static final long serialVersionUID = -8734577898966155218L;

   public MRUAlgorithmConfig()
   {
      evictionAlgorithmClassName = MRUAlgorithm.class.getName();
      // We require that maxNodes is set
      setMaxNodes(-1);
   }

   public MRUAlgorithmConfig(int maxNodes)
   {
      evictionAlgorithmClassName = MRUAlgorithm.class.getName();
      setMaxNodes(maxNodes);
   }

   /**
    * Requires a positive maxNodes value or ConfigurationException
    * is thrown.
    */
   @Override
   public void validate() throws ConfigurationException
   {
      super.validate();
      if (getMaxNodes() < 0)
         throw new ConfigurationException("maxNodes not configured");
   }

   @Override
   public String toString()
   {
      StringBuilder str = new StringBuilder();
      str.append("MRUAlgorithmConfig: ").
            append(" maxNodes =").append(getMaxNodes());
      return str.toString();
   }

   @Override
   public boolean equals(Object obj)
   {
      return (obj instanceof MRUAlgorithmConfig && super.equals(obj));
   }

   @Override
   public void reset()
   {
      super.reset();
      setMaxNodes(-1);
      evictionAlgorithmClassName = MRUAlgorithm.class.getName();
   }

   @Override
   public MRUAlgorithmConfig clone() throws CloneNotSupportedException
   {
      return (MRUAlgorithmConfig) super.clone();
   }

}