package org.jboss.cache.factories;

import org.jboss.cache.commands.CommandsFactory;
import org.jboss.cache.commands.CommandsFactoryImpl;
import org.jboss.cache.commands.OptimisticCommandsFactoryImpl;
import org.jboss.cache.commands.PessimisticCommandsFactoryImpl;
import org.jboss.cache.config.ConfigurationException;
import org.jboss.cache.factories.annotations.DefaultFactoryFor;

/**
 * COnstructs commands factory
 *
 * @author Manik Surtani (<a href="mailto:manik@jboss.org">manik@jboss.org</a>)
 * @since 3.0
 */
@DefaultFactoryFor(classes = CommandsFactory.class)
public class CommandsMetaFactory extends ComponentFactory
{
   @SuppressWarnings("deprecation")
   protected <T> T construct(Class<T> componentType)
   {
      switch (configuration.getNodeLockingScheme())
      {
         case MVCC:
            return componentType.cast(new CommandsFactoryImpl());
         case OPTIMISTIC:
            return componentType.cast(new OptimisticCommandsFactoryImpl());
         case PESSIMISTIC:
            return componentType.cast(new PessimisticCommandsFactoryImpl());
      }
      throw new ConfigurationException("Unknown locking scheme " + configuration.getNodeLockingScheme());
   }
}
